import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis,
  timeFormat
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// Formating the Y-axis
const yAxisTickFormat = number =>
    format('.2s')(number)

// Formating the X-axis
const xAxisTickFormat = timeFormat("%Y");

// Area Chart
const render = data => {
  
  // Years is the value on X-axis
  const xValue = d => d.Years;
  const xAxisLabel = 'Year';
  
  // Number of sales is the value on Y-axis
  const yValue = d => d.Bike_Sales;
  const yAxisLabel = 'Number of Sales';
  
  const margin = { top: 10, right: 40, bottom: 88, left: 150 };
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
// Creating a scale for X axis based on the xValue from the data
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
// Create a scale for Y axis based on the yValue from the data
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
 // The position of SVG on the graph
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15)
  	.tickFormat(xAxisTickFormat);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(yAxisTickFormat);
  
  //  Y-axis
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -93)
      .attr('x', -innerHeight / 2)
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  //  X-axis
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
 // Generates the line based on the xValues and yValues
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
 // Generates the area under the curve
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
  g.append('path')
  	.attr('class', 'line-chart')
  	.attr('d', lineGenerator(data));
  
  g.append('path')
  	.attr('class', 'area-chart')
  	.attr('d', areaGenerator(data));
  
 
};

// Data set URI
csv('https://gist.githubusercontent.com/ajayrao0/35ae2f9d3b3c41e7a20baa0d4482045a/raw/07c8fbac0c6822c7e9a3eef107491fdb0188175c/ice7q4.csv')
  .then(data => {
    data.forEach(d => {
      d.Bike_Sales = +d.Bike_Sales;
      d.Years= new Date(d.Years);
    });
    render(data);
  });